@@ -175,6 +175,12 @@ double gentime_dens(gentime *in, int t, int kappa_i){
       /* exit(1); */
     }
 
+    /* error if requested time is <= 0 */
+    if(t <= 0){
+      /* warning("\n[in: structures.c->gentime_dens]\nTrying to get density for %d time units (min: 1).\n", t); */
+      return 0.0;
+    }
+
     /* otherwise fetch density value */
     double out=mat_double_ij(in->dens, kappa_i-1, t);
     return out;
@@ -190,6 +196,12 @@ double colltime_dens(gentime *in, int t){
 	return 0.0;
     }
 
+    /* error if requested time is <= 0 */
+    if(t <= 0){
+      /* warning("\n[in: structures.c->colltime_dens]\nTrying to get density for %d time units (min: 1).\n", t); */
+      return 0.0;
+    }
+
     /* otherwise fetch density value */
     double out=vec_double_i(in->collTime, t);
     return out;
@@ -215,6 +227,8 @@ param *alloc_param(int n){
     /* fill in integers */
     out->n = n;
     out->kappa_temp = 0;
+    out->mut_model = 0;
+    out->spa_model = 0;
 
     /* allocates vectors of integers */
     out->Tinf = alloc_vec_int(n);
@@ -228,6 +242,10 @@ param *alloc_param(int n){
     out->pi = 1.0;
     out->pi_param1 = 0.0;
     out->pi_param2 = 0.0;
+    out->spa_param1 = 0.0;
+    out->spa_param2 = 0.0;
+    out->spa_param1_prior = 0.0;
+    out->spa_param2_prior = 0.0;
     out->outlier_threshold = 1000.0;
     /* out->phi = 0.1; */
     /* out->phi_param1 = 0.0; */
@@ -257,7 +275,7 @@ void print_param(param *in){
     print_vec_int(in->alpha);
     Rprintf("\n= Kappa_i (generations from nearest ancestor) =\n");
     print_vec_int(in->kappa);
-    Rprintf("\n= mu1, mu2, gamma (transi, transver, coef, prior mu1) =\n");
+    Rprintf("\n= mu1, mu2, gamma (mutation1, mutation2, coef, prior mu1) =\n");
     Rprintf("%.5f   %.5f   %.5f   %.5f", in->mu1, in->gamma*in->mu1, in->gamma, in->mu1_prior);
     Rprintf("\n= pi (proportion of observed cases) =\n");
     Rprintf("%.5f", in->pi);
@@ -265,6 +283,12 @@ void print_param(param *in){
     Rprintf("%.5f  %.5f", in->pi_param1, in->pi_param2);
     Rprintf("\n= threshold used in imported case detection =\n");
     Rprintf("%.2f", in->outlier_threshold);
+    Rprintf("\n= genetic model used =\n");
+    Rprintf("%d", in->mut_model);
+    Rprintf("\n= spatial model used (parameters) =\n");
+    Rprintf("%d (param1=%.5f, param2=%.5f)", in->spa_model, in->spa_param1, in->spa_param2);
+    Rprintf("\n= parameters of the spatial priors =\n");
+    Rprintf("(mean prior param1=%.5f, mean prior param2=%.5f)", in->spa_param1_prior, in->spa_param2_prior);
 
     /* Rprintf("\n= phi (proportion of external cases) =\n"); */
     /* Rprintf("%.5f", in->phi); */
@@ -285,10 +309,14 @@ void copy_param(param *in, param *out){
     out->pi = in->pi;
     out->pi_param1 = in->pi_param1;
     out->pi_param2 = in->pi_param2;
+    out->spa_param1 = in->spa_param1;
+    out->spa_param2 = in->spa_param2;
+    out->spa_param1_prior = in->spa_param1_prior;
+    out->spa_param2_prior = in->spa_param2_prior;
     out->outlier_threshold = in->outlier_threshold;
-    /* out->phi = in->phi; */
-    /* out->phi_param1 = in->phi_param1; */
-    /* out->phi_param2 = in->phi_param2; */
+    out->mut_model = in->mut_model;
+    out->spa_model = in->spa_model;
+
     copy_vec_int(in->Tinf,out->Tinf);
     copy_vec_int(in->alpha,out->alpha);
     copy_vec_int(in->kappa,out->kappa);
@@ -318,13 +346,13 @@ mcmc_param *alloc_mcmc_param(int n){
     }
 
     /* DETERMINE THE NUMBER OF Tinf */
-    /* set to N/3, minimum 1 */
+    /* set to N/2, minimum 1 */
     out->n_move_Tinf = (int) n/2;
     out->n_move_Tinf = out->n_move_Tinf < 1 ? 1 : out->n_move_Tinf;
 
 
     /* DETERMINE THE NUMBER OF KAPPA AND ALPHA TO MOVE */
-    /* set to N/3, minimum 1 */
+    /* set to N/2, minimum 1 */
     out->n_move_alpha = (int) n/2;
     out->n_move_alpha = out->n_move_alpha < 1 ? 1 : out->n_move_alpha;
     out->n_move_kappa = out->n_move_alpha;
@@ -344,6 +372,10 @@ mcmc_param *alloc_mcmc_param(int n){
     out->n_reject_mu1 = 0;
     out->n_accept_gamma = 0;
     out->n_reject_gamma = 0;
+    out->n_accept_spa1 = 0;
+    out->n_reject_spa1 = 0;
+    out->n_accept_spa2 = 0;
+    out->n_reject_spa2 = 0;
     out->n_accept_Tinf = 0;
     out->n_reject_Tinf = 0;
     out->n_accept_alpha = 0;
@@ -361,6 +393,8 @@ mcmc_param *alloc_mcmc_param(int n){
     /* out->move_alpha = TRUE; */
     /* out->move_kappa = TRUE; */
     out->move_pi = TRUE;
+    out->move_spa1 = TRUE;
+    out->move_spa2 = TRUE;
     /* out->move_phi = TRUE; */
     out->burnin=0;
     out->find_import_at=10000;
@@ -370,6 +404,8 @@ mcmc_param *alloc_mcmc_param(int n){
     out->sigma_mu1 = 0.0;
     out->sigma_gamma = 0.0;
     out->sigma_pi = 0.0;
+    out->sigma_spa1 = 0.0;
+    out->sigma_spa2 = 0.0;
     /* out->sigma_phi = 0.0; */
     out->lambda_Tinf = 0.0;
 
@@ -400,6 +436,8 @@ void print_mcmc_param(mcmc_param *in){
     Rprintf("\nsigma for mu1: %.10f",in->sigma_mu1);
     Rprintf("\nsigma for gamma: %.10f",in->sigma_gamma);
     Rprintf("\nsigma for pi: %.10f",in->sigma_pi);
+    Rprintf("\nsigma for spa1: %.10f",in->sigma_spa1);
+    Rprintf("\nsigma for spa2: %.10f",in->sigma_spa2);
     /* Rprintf("\nsigma for phi: %.10f",in->sigma_phi); */
     Rprintf("\nlambda for Tinf: %.10f",in->lambda_Tinf);
     Rprintf("\nnb moves for Tinf: %d",in->n_move_Tinf);
@@ -412,6 +450,10 @@ void print_mcmc_param(mcmc_param *in){
 
     Rprintf("\npi: nb. accepted: %d   nb. rejected: %d   (acc/rej ratio:%.3f)", in->n_accept_pi, in->n_reject_pi, (double) in->n_accept_pi / in->n_reject_pi);
 
+    Rprintf("\nspa1: nb. accepted: %d   nb. rejected: %d   (acc/rej ratio:%.3f)", in->n_accept_spa1, in->n_reject_spa1, (double) in->n_accept_spa1 / in->n_reject_spa1);
+
+    Rprintf("\nspa2: nb. accepted: %d   nb. rejected: %d   (acc/rej ratio:%.3f)", in->n_accept_spa2, in->n_reject_spa2, (double) in->n_accept_spa2 / in->n_reject_spa2);
+
     /* Rprintf("\nphi: nb. accepted: %d   nb. rejected: %d   (acc/rej ratio:%.3f)", in->n_accept_phi, in->n_reject_phi, (double) in->n_accept_phi / in->n_reject_phi); */
 
     Rprintf("\nTinf: nb. accepted: %d   nb. rejected: %d   (acc/rej ratio:%.3f)", in->n_accept_Tinf, in->n_reject_Tinf, (double) in->n_accept_Tinf / in->n_reject_Tinf);
@@ -440,6 +482,8 @@ void print_mcmc_param(mcmc_param *in){
     if(in->tune_mu1) Rprintf("mu1 ");
     if(in->tune_gamma) Rprintf("gamma ");
     if(in->tune_pi) Rprintf("pi ");
+    if(in->tune_spa1) Rprintf("spa1");
+    if(in->tune_spa2) Rprintf("spa2");
     /* if(in->tune_phi) Rprintf("phi "); */
     Rprintf("\nTuning stopped at step %d\n", in->step_notune);
 
@@ -449,6 +493,8 @@ void print_mcmc_param(mcmc_param *in){
     /* if(in->move_kappa) Rprintf("kappa "); */
     if(in->move_Tinf) Rprintf("Tinf ");
     if(in->move_pi) Rprintf("pi ");
+    if(in->move_spa1) Rprintf("spa1 ");
+    if(in->move_spa2) Rprintf("spa2 ");
     /* if(in->move_phi) Rprintf("phi "); */
     Rprintf("\nMove alpha_i:");
     print_vec_double(in->move_alpha);
@@ -479,33 +525,41 @@ void copy_mcmc_param(mcmc_param *in, mcmc_param *out){
     out->n_reject_gamma = in->n_reject_gamma;
     out->n_accept_pi = in->n_accept_pi;
     out->n_reject_pi = in->n_reject_pi;
-    /* out->n_accept_phi = in->n_accept_phi; */
-    /* out->n_reject_phi = in->n_reject_phi; */
+    out->n_accept_spa1 = in->n_accept_spa1;
+    out->n_reject_spa1 = in->n_reject_spa1;
+    out->n_accept_spa2 = in->n_accept_spa2;
+    out->n_reject_spa2 = in->n_reject_spa2;
     out->n_accept_Tinf = in->n_accept_Tinf;
     out->n_reject_Tinf = in->n_reject_Tinf;
     out->n_accept_alpha = in->n_accept_alpha;
     out->n_reject_alpha = in->n_reject_alpha;
     out->n_accept_kappa = in->n_accept_kappa;
     out->n_reject_kappa = in->n_reject_kappa;
+
     out->n_move_Tinf = in->n_move_Tinf;
     out->n_move_alpha = in->n_move_alpha;
     out->n_move_kappa = in->n_move_kappa;
+
     out->sigma_mu1 = in->sigma_mu1;
     out->sigma_gamma = in->sigma_gamma;
     out->lambda_Tinf = in->lambda_Tinf;
     out->sigma_pi = in->sigma_pi;
-    /* out->sigma_phi = in->sigma_phi; */
+    out->sigma_spa1 = in->sigma_spa1;
+    out->sigma_spa2 = in->sigma_spa2;
     out->n_like_zero = in->n_like_zero;
+
     out->tune_all = in->tune_all;
     out->tune_mu1 = in->tune_mu1;
     out->tune_gamma = in->tune_gamma;
     out->tune_pi = in->tune_pi;
-    /* out->tune_phi = in->tune_phi; */
+    out->tune_spa1 = in->tune_spa1;
+    out->tune_spa2 = in->tune_spa2;
     out->step_notune = in->step_notune;
+
     out->move_mut = in->move_mut;
-    /* out->move_kappa = in->move_kappa; */
     out->move_pi = in->move_pi;
-    /* out->move_phi = in->move_phi; */
+    out->move_spa1 = in->move_spa1;
+    out->move_spa2 = in->move_spa2;
     out->burnin = in->burnin;
     out->find_import_at = in->find_import_at;
     out->find_import = in->find_import;
